@@ -39,6 +39,7 @@
 #include <sys/types.h>
 #include <stdarg.h>
 #include <getopt.h>
+#include <unistd.h>
 
 #ifndef IGT_LOG_DOMAIN
 #define IGT_LOG_DOMAIN (NULL)
@@ -231,6 +232,7 @@ void __igt_subtest_group_restore(int);
 #define igt_main \
 	static void igt_tokencat(__real_main, __LINE__)(void); \
 	int main(int argc, char **argv) { \
+		sync(); \
 		igt_subtest_init_parse_opts(&argc, argv, NULL, NULL, NULL, \
 					    NULL, NULL); \
 		igt_tokencat(__real_main, __LINE__)(); \
@@ -271,6 +273,7 @@ void igt_simple_init_parse_opts(int *argc, char **argv,
 #define igt_simple_main \
 	static void igt_tokencat(__real_main, __LINE__)(void); \
 	int main(int argc, char **argv) { \
+		sync(); \
 		igt_simple_init_parse_opts(&argc, argv, NULL, NULL, NULL, \
 					   NULL, NULL); \
 		igt_tokencat(__real_main, __LINE__)(); \
@@ -296,6 +299,17 @@ void __igt_fail_assert(const char *domain, const char *file,
 	__attribute__((noreturn));
 void igt_exit(void) __attribute__((noreturn));
 
+/**
+ * igt_ignore_warn:
+ * @expr: condition to ignore
+ *
+ *
+ * Stops the compiler warning about an unused return value.
+ */
+static inline void igt_ignore_warn(bool value)
+{
+}
+
 /**
  * igt_assert:
  * @expr: condition to test
@@ -391,6 +405,24 @@ void igt_exit(void) __attribute__((noreturn));
 				  "error: %#x " #ncmp " %#x\n", __n1, __n2); \
 	} while (0)
 
+/**
+ * igt_assert_cmps64:
+ * @n1: first value
+ * @cmp: compare operator
+ * @ncmp: negated version of @cmp
+ * @n2: second value
+ *
+ * Like igt_assert_cmpuint(), but for larger signed ints.
+ */
+#define igt_assert_cmps64(n1, cmp, ncmp, n2) \
+	do { \
+		int64_t __n1 = (n1), __n2 = (n2); \
+		if (__n1 cmp __n2) ; else \
+		__igt_fail_assert(IGT_LOG_DOMAIN, __FILE__, __LINE__, __func__, \
+				  #n1 " " #cmp " " #n2, \
+				  "error: %lld " #ncmp " %lld\n", (long long)__n1, (long long)__n2); \
+	} while (0)
+
 /**
  * igt_assert_cmpu64:
  * @n1: first value
@@ -449,6 +481,15 @@ void igt_exit(void) __attribute__((noreturn));
  */
 #define igt_assert_eq_u32(n1, n2) igt_assert_cmpuint(n1, ==, !=, n2)
 
+/**
+ * igt_assert_eq_s64:
+ * @n1: first integer
+ * @n2: second integer
+ *
+ * Like igt_assert_eq_u32(), but for int64_t.
+ */
+#define igt_assert_eq_s64(n1, n2) igt_assert_cmps64(n1, ==, !=, n2)
+
 /**
  * igt_assert_eq_u64:
  * @n1: first integer
@@ -709,7 +750,16 @@ void igt_skip_on_simulation(void);
 
 extern const char *igt_interactive_debug;
 
-/* structured logging */
+/**
+ * igt_log_level:
+ * @IGT_LOG_DEBUG: debug information, not printed by default
+ * @IGT_LOG_INFO: informational message, printed by default
+ * @IGT_LOG_WARN: non-fatal warnings which should be treated as test failures
+ * @IGT_LOG_CRITICAL: critical errors which lead to immediate termination of tests
+ * @IGT_LOG_NONE: unused
+ *
+ * Log levels used by functions like igt_log().
+ */
 enum igt_log_level {
 	IGT_LOG_DEBUG,
 	IGT_LOG_INFO,
