@@ -76,7 +76,6 @@ static double nop_on_ring(int fd, uint32_t handle, unsigned ring_id,
 		execbuf.flags = ring_id;
 		gem_execbuf(fd, &execbuf);
 	}
-	gem_sync(fd, handle);
 	intel_detect_and_clear_missed_interrupts(fd);
 
 	count = 0;
@@ -88,8 +87,6 @@ static double nop_on_ring(int fd, uint32_t handle, unsigned ring_id,
 		count += 1024;
 		clock_gettime(CLOCK_MONOTONIC, &now);
 	} while (elapsed(&start, &now) < timeout);
-	gem_sync(fd, handle);
-	clock_gettime(CLOCK_MONOTONIC, &now);
 	igt_assert_eq(intel_detect_and_clear_missed_interrupts(fd), 0);
 
 	*out = count;
@@ -120,11 +117,76 @@ static bool ignore_engine(int fd, unsigned engine)
 	return false;
 }
 
-static void all(int fd, uint32_t handle, int timeout)
+static void parallel(int fd, uint32_t handle, int timeout)
 {
 	struct drm_i915_gem_execbuffer2 execbuf;
 	struct drm_i915_gem_exec_object2 obj;
-	struct timespec start, now;
+	unsigned engines[16];
+	const char *names[16];
+	unsigned nengine;
+	unsigned engine;
+	unsigned long count;
+	double time, sum;
+
+	sum = 0;
+	nengine = 0;
+	for_each_engine(fd, engine) {
+		if (ignore_engine(fd, engine))
+			continue;
+
+		engines[nengine] = engine;
+		names[nengine] = e__->name;
+		nengine++;
+
+		time = nop_on_ring(fd, handle, engine, 1, &count) / count;
+		sum += time;
+		igt_debug("%s: %.3fus\n", e__->name, 1e6*time);
+	}
+	igt_require(nengine);
+	igt_info("average (individually): %.3fus\n", sum/nengine*1e6);
+
+	memset(&obj, 0, sizeof(obj));
+	obj.handle = handle;
+
+	memset(&execbuf, 0, sizeof(execbuf));
+	execbuf.buffers_ptr = (uintptr_t)&obj;
+	execbuf.buffer_count = 1;
+	execbuf.flags |= LOCAL_I915_EXEC_HANDLE_LUT;
+	execbuf.flags |= LOCAL_I915_EXEC_NO_RELOC;
+	if (__gem_execbuf(fd, &execbuf)) {
+		execbuf.flags = 0;
+		gem_execbuf(fd, &execbuf);
+	}
+	intel_detect_and_clear_missed_interrupts(fd);
+
+	igt_fork(child, nengine) {
+		struct timespec start, now;
+
+		execbuf.flags &= ~ENGINE_FLAGS;
+		execbuf.flags |= engines[child];
+
+		count = 0;
+		clock_gettime(CLOCK_MONOTONIC, &start);
+		do {
+			for (int loop = 0; loop < 1024; loop++)
+				gem_execbuf(fd, &execbuf);
+			count += 1024;
+			clock_gettime(CLOCK_MONOTONIC, &now);
+		} while (elapsed(&start, &now) < timeout);
+		time = elapsed(&start, &now) / count;
+		igt_info("%s: %ld cycles, %.3fus\n", names[child], count, 1e6*time);
+	}
+
+	igt_waitchildren();
+	igt_assert_eq(intel_detect_and_clear_missed_interrupts(fd), 0);
+
+}
+
+static void series(int fd, uint32_t handle, int timeout)
+{
+	struct drm_i915_gem_execbuffer2 execbuf;
+	struct drm_i915_gem_exec_object2 obj;
+	struct timespec start, now, sync;
 	unsigned engines[16];
 	unsigned nengine;
 	unsigned engine;
@@ -148,8 +210,8 @@ static void all(int fd, uint32_t handle, int timeout)
 		engines[nengine++] = engine;
 	}
 	igt_require(nengine);
-	igt_info("Maximum execution latency on %s, %.3fus, total %.3fus per cycle\n",
-		 name, max*1e6, sum*1e6);
+	igt_info("Maximum execution latency on %s, %.3fus, min %.3fus, total %.3fus per cycle, average %.3fus\n",
+		 name, max*1e6, min*1e6, sum*1e6, sum/nengine*1e6);
 
 	memset(&obj, 0, sizeof(obj));
 	obj.handle = handle;
@@ -163,7 +225,6 @@ static void all(int fd, uint32_t handle, int timeout)
 		execbuf.flags = 0;
 		gem_execbuf(fd, &execbuf);
 	}
-	gem_sync(fd, handle);
 	intel_detect_and_clear_missed_interrupts(fd);
 
 	count = 0;
@@ -180,21 +241,30 @@ static void all(int fd, uint32_t handle, int timeout)
 		clock_gettime(CLOCK_MONOTONIC, &now);
 	} while (elapsed(&start, &now) < timeout); /* Hang detection ~120s */
 	gem_sync(fd, handle);
-	clock_gettime(CLOCK_MONOTONIC, &now);
+	clock_gettime(CLOCK_MONOTONIC, &sync);
+	igt_debug("sync time: %.3fus\n", elapsed(&now, &sync)*1e6);
 	igt_assert_eq(intel_detect_and_clear_missed_interrupts(fd), 0);
 
 	time = elapsed(&start, &now) / count;
-	igt_info("All (%d engines): %'lu cycles, average %.3fus per cycle\n",
-		 nengine, count, 1e6*time);
+	igt_info("All (%d engines): %'lu cycles, average %.3fus per cycle [expected %.3fus]\n",
+		 nengine, count, 1e6*time, 1e6*((max-min)/nengine+min));
 
-	/* The rate limiting step is how fast the slowest engine can
-	 * its queue of requests, if we wait upon a full ring all dispatch
-	 * is frozen. So in general we cannot go faster than the slowest
-	 * engine, but we should equally not go any slower.
+	/* The rate limiting step should be how fast the slowest engine can
+	 * execute its queue of requests, as when we wait upon a full ring all
+	 * dispatch is frozen. So in general we cannot go faster than the
+	 * slowest engine (but as all engines are in lockstep, they should all
+	 * be executing in parallel and so the average should be max/nengines),
+	 * but we should equally not go any slower.
+	 *
+	 * However, that depends upon being able to submit fast enough, and
+	 * that in turns depends upon debugging turned off and no bottlenecks
+	 * within the driver. We cannot assert that we hit ideal conditions
+	 * across all engines, so we only look for an outrageous error
+	 * condition.
 	 */
-	igt_assert_f(time < max + 10*min/9, /* ensure parallel execution */
-		     "Average time (%.3fus) exceeds expecation for parallel execution (min %.3fus, max %.3fus; limit set at %.3fus)\n",
-		     1e6*time, 1e6*min, 1e6*max, 1e6*(max + 10*min/9));
+	igt_assert_f(time < 2*sum,
+		     "Average time (%.3fus) exceeds expectation for parallel execution (min %.3fus, max %.3fus; limit set at %.3fus)\n",
+		     1e6*time, 1e6*min, 1e6*max, 1e6*sum*2);
 }
 
 static void print_welcome(int fd)
@@ -219,7 +289,7 @@ static void print_welcome(int fd)
 	}
 
 	active = igt_sysfs_get_boolean(dir, "semaphores");
-	igt_info("Using Legacy submission %s\n",
+	igt_info("Using Legacy submission%s\n",
 		 active ? ", with semaphores" : "");
 
 out:
@@ -244,15 +314,21 @@ igt_main
 		igt_fork_hang_detector(device);
 	}
 
-	igt_subtest("basic")
-		all(device, handle, 10);
+	igt_subtest("basic-series")
+		series(device, handle, 5);
+
+	igt_subtest("basic-parallel")
+		parallel(device, handle, 5);
 
 	for (e = intel_execution_engines; e->name; e++)
 		igt_subtest_f("%s", e->name)
 			single(device, handle, e->exec_id | e->flags, e->name);
 
-	igt_subtest("all")
-		all(device, handle, 150);
+	igt_subtest("series")
+		series(device, handle, 150);
+
+	igt_subtest("parallel")
+		parallel(device, handle, 150);
 
 	igt_fixture {
 		igt_stop_hang_detector();
