@@ -24,6 +24,7 @@
 
 #define _GNU_SOURCE
 #include <sched.h>
+#include <sys/poll.h>
 
 #include "igt.h"
 #include "igt_rand.h"
@@ -450,7 +451,195 @@ static unsigned get_vblank(int fd, int pipe, unsigned flags)
 	return vbl.reply.sequence;
 }
 
-static void basic_flip_vs_cursor(igt_display_t *display, enum flip_test mode, int nloops)
+enum basic_flip_cursor {
+	FLIP_BEFORE_CURSOR,
+	FLIP_AFTER_CURSOR
+};
+
+static uint32_t *make_busy(int fd, uint32_t target)
+{
+	const int gen = intel_gen(intel_get_drm_devid(fd));
+	struct drm_i915_gem_exec_object2 obj[2];
+	struct drm_i915_gem_relocation_entry reloc[2];
+	struct drm_i915_gem_execbuffer2 execbuf;
+	uint32_t *batch;
+	int i;
+
+	memset(&execbuf, 0, sizeof(execbuf));
+	execbuf.buffers_ptr = (uintptr_t)obj;
+	execbuf.buffer_count = 2;
+
+	memset(obj, 0, sizeof(obj));
+	obj[0].handle = target;
+	obj[1].handle = gem_create(fd, 4096);
+	batch = gem_mmap__wc(fd, obj[1].handle, 0, 4096, PROT_WRITE);
+	gem_set_domain(fd, obj[1].handle,
+			I915_GEM_DOMAIN_GTT, I915_GEM_DOMAIN_GTT);
+
+
+	obj[1].relocs_ptr = (uintptr_t)reloc;
+	obj[1].relocation_count = 2;
+	memset(reloc, 0, sizeof(reloc));
+
+	reloc[0].target_handle = obj[1].handle; /* recurse */
+	reloc[0].presumed_offset = 0;
+	reloc[0].offset = sizeof(uint32_t);
+	reloc[0].delta = 0;
+	reloc[0].read_domains = I915_GEM_DOMAIN_COMMAND;
+	reloc[0].write_domain = 0;
+
+	reloc[1].target_handle = target;
+	reloc[1].presumed_offset = 0;
+	reloc[1].offset = 1024;
+	reloc[1].delta = 0;
+	reloc[1].read_domains = I915_GEM_DOMAIN_COMMAND;
+	reloc[1].write_domain = I915_GEM_DOMAIN_COMMAND;
+
+	i = 0;
+	batch[i] = MI_BATCH_BUFFER_START;
+	if (gen >= 8) {
+		batch[i] |= 1 << 8 | 1;
+		batch[++i] = 0;
+		batch[++i] = 0;
+	} else if (gen >= 6) {
+		batch[i] |= 1 << 8;
+		batch[++i] = 0;
+	} else {
+		batch[i] |= 2 << 6;
+		batch[++i] = 0;
+		if (gen < 4) {
+			batch[i] |= 1;
+			reloc[0].delta = 1;
+		}
+	}
+	i++;
+
+	gem_execbuf(fd, &execbuf);
+	gem_close(fd, obj[1].handle);
+
+	return batch;
+}
+
+static void cancel_busy(uint32_t *busy)
+{
+	*busy = MI_BATCH_BUFFER_END;
+	munmap(busy, 4096);
+}
+
+static uint32_t *
+make_fb_busy(int fd, const struct igt_fb *fb)
+{
+	uint32_t *busy;
+
+	busy = make_busy(fd, fb->gem_handle);
+	igt_assert(gem_bo_busy(fd, fb->gem_handle));
+
+	return busy;
+}
+
+static void finish_fb_busy(uint32_t *busy)
+{
+	cancel_busy(busy);
+}
+
+#define BASIC_BUSY 0x1
+
+static void basic_flip_cursor(igt_display_t *display,
+			      enum flip_test mode,
+			      enum basic_flip_cursor order,
+			      unsigned flags)
+{
+	struct drm_mode_cursor arg[2];
+	struct drm_event_vblank vbl;
+	struct igt_fb fb_info, cursor_fb, cursor_fb2, argb_fb;
+	unsigned vblank_start;
+	enum pipe pipe = find_connected_pipe(display, false);
+	uint32_t *busy;
+
+	if (mode >= flip_test_atomic)
+		igt_require(display->is_atomic);
+
+	igt_require(set_fb_on_crtc(display, pipe, &fb_info));
+
+	igt_create_color_fb(display->drm_fd, 64, 64, DRM_FORMAT_ARGB8888, 0, 1., 1., 1., &cursor_fb);
+	set_cursor_on_pipe(display, pipe, &cursor_fb);
+	populate_cursor_args(display, pipe, arg, &cursor_fb);
+
+	prepare_flip_test(display, mode, pipe, pipe, arg, &fb_info, &argb_fb, &cursor_fb2);
+
+	igt_display_commit2(display, display->is_atomic ? COMMIT_ATOMIC : COMMIT_LEGACY);
+
+	/* Quick sanity check that we can update a cursor in a single vblank */
+	vblank_start = get_vblank(display->drm_fd, pipe, DRM_VBLANK_NEXTONMISS);
+	igt_assert_eq(get_vblank(display->drm_fd, pipe, 0), vblank_start);
+	do_ioctl(display->drm_fd, DRM_IOCTL_MODE_CURSOR, &arg[0]);
+	igt_assert_eq(get_vblank(display->drm_fd, pipe, 0), vblank_start);
+
+	/* Bind the cursor first to warm up */
+	do_ioctl(display->drm_fd, DRM_IOCTL_MODE_CURSOR, &arg[0]);
+
+	/* Start with a synchronous query to align with the vblank */
+	vblank_start = get_vblank(display->drm_fd, pipe, DRM_VBLANK_NEXTONMISS);
+
+	busy = NULL;
+	if (flags & BASIC_BUSY)
+		busy = make_fb_busy(display->drm_fd, &fb_info);
+
+	switch (order) {
+	case FLIP_BEFORE_CURSOR:
+		switch (mode) {
+		default:
+			flip_nonblocking(display, pipe, mode >= flip_test_atomic, &fb_info);
+			break;
+		case flip_test_atomic_transitions:
+		case flip_test_atomic_transitions_varying_size:
+			transition_nonblocking(display, pipe, &fb_info, &argb_fb, 0);
+			break;
+		}
+		igt_assert_eq(get_vblank(display->drm_fd, pipe, 0), vblank_start);
+
+		do_ioctl(display->drm_fd, DRM_IOCTL_MODE_CURSOR, &arg[0]);
+		igt_assert_eq(get_vblank(display->drm_fd, pipe, 0), vblank_start);
+		break;
+
+	case FLIP_AFTER_CURSOR:
+		do_ioctl(display->drm_fd, DRM_IOCTL_MODE_CURSOR, &arg[0]);
+		igt_assert_eq(get_vblank(display->drm_fd, pipe, 0), vblank_start);
+
+		switch (mode) {
+		default:
+			flip_nonblocking(display, pipe, mode >= flip_test_atomic, &fb_info);
+			break;
+		case flip_test_atomic_transitions:
+		case flip_test_atomic_transitions_varying_size:
+			transition_nonblocking(display, pipe, &fb_info, &argb_fb, 0);
+			break;
+		}
+		igt_assert_eq(get_vblank(display->drm_fd, pipe, 0), vblank_start);
+	}
+
+	if (busy) {
+		struct pollfd pfd = { display->drm_fd, POLLIN };
+		igt_assert(poll(&pfd, 1, 0) == 0);
+		finish_fb_busy(busy);
+	}
+
+	igt_set_timeout(1, "Stuck page flip");
+	igt_ignore_warn(read(display->drm_fd, &vbl, sizeof(vbl)));
+	igt_assert_eq(get_vblank(display->drm_fd, pipe, 0), vblank_start + 1);
+	igt_reset_timeout();
+
+	do_cleanup_display(display);
+	igt_remove_fb(display->drm_fd, &fb_info);
+	igt_remove_fb(display->drm_fd, &cursor_fb);
+
+	if (argb_fb.gem_handle)
+		igt_remove_fb(display->drm_fd, &argb_fb);
+	if (cursor_fb2.gem_handle)
+		igt_remove_fb(display->drm_fd, &cursor_fb2);
+}
+
+static void flip_vs_cursor(igt_display_t *display, enum flip_test mode, int nloops)
 {
 	struct drm_mode_cursor arg[2];
 	struct drm_event_vblank vbl;
@@ -555,6 +744,99 @@ static bool skip_on_unsupported_nonblocking_modeset(igt_display_t *display)
 	return false;
 }
 
+static void nonblocking_modeset_vs_cursor(igt_display_t *display, int loops)
+{
+	struct igt_fb fb_info, cursor_fb;
+	igt_output_t *output;
+	enum pipe pipe = find_connected_pipe(display, false);
+	struct drm_mode_cursor arg;
+	bool skip_test;
+	igt_plane_t *cursor = NULL, *plane;
+
+	igt_require(display->is_atomic);
+	igt_require((output = set_fb_on_crtc(display, pipe, &fb_info)));
+	igt_create_color_fb(display->drm_fd, 64, 64, DRM_FORMAT_ARGB8888, 0, 1., 1., 1., &cursor_fb);
+	set_cursor_on_pipe(display, pipe, &cursor_fb);
+	populate_cursor_args(display, pipe, &arg, &cursor_fb);
+	arg.flags |= DRM_MODE_CURSOR_BO;
+
+	for_each_plane_on_pipe(display, pipe, plane) {
+		if (!plane->is_cursor)
+			continue;
+
+		cursor = plane;
+		break;
+	}
+
+	igt_skip_on(!cursor);
+
+	if ((skip_test = skip_on_unsupported_nonblocking_modeset(display)))
+		goto cleanup;
+
+	/*
+	 * Start disabled, because skip_on_unsupported_nonblocking_modeset
+	 * will have enabled this pipe. No way around it, since the first
+	 * atomic commit may be unreliable with amount of events sent.
+	 */
+	igt_output_set_pipe(output, PIPE_NONE);
+	igt_display_commit2(display, COMMIT_ATOMIC);
+
+	while (loops--) {
+		unsigned flags;
+		struct pollfd pfd = { display->drm_fd, POLLIN };
+		struct drm_event_vblank vbl;
+
+		flags = DRM_MODE_ATOMIC_ALLOW_MODESET;
+		flags |= DRM_MODE_ATOMIC_NONBLOCK;
+		flags |= DRM_MODE_PAGE_FLIP_EVENT;
+
+		/*
+		 * Test that a cursor update after a nonblocking modeset
+		 * works as intended. It should block until the modeset completes.
+		 */
+
+		igt_output_set_pipe(output, pipe);
+		igt_plane_set_fb(cursor, NULL);
+		igt_display_commit_atomic(display, flags, NULL);
+
+		igt_assert_eq(0, poll(&pfd, 1, 0));
+		igt_assert_eq(0, pfd.revents);
+
+		do_ioctl(display->drm_fd, DRM_IOCTL_MODE_CURSOR, &arg);
+
+		igt_assert_eq(1, poll(&pfd, 1, 0));
+		igt_assert_eq(POLLIN, pfd.revents);
+
+		igt_set_timeout(1, "Stuck page flip");
+		igt_ignore_warn(read(display->drm_fd, &vbl, sizeof(vbl)));
+		igt_reset_timeout();
+
+		igt_output_set_pipe(output, PIPE_NONE);
+		igt_display_commit_atomic(display, flags, NULL);
+
+		igt_assert_eq(0, poll(&pfd, 1, 0));
+		igt_assert_eq(0, pfd.revents);
+
+		/* Same for cursor on disabled crtc. */
+		do_ioctl(display->drm_fd, DRM_IOCTL_MODE_CURSOR, &arg);
+
+		igt_assert_eq(1, poll(&pfd, 1, 0));
+		igt_assert_eq(POLLIN, pfd.revents);
+
+		igt_set_timeout(1, "Stuck page flip");
+		igt_ignore_warn(read(display->drm_fd, &vbl, sizeof(vbl)));
+		igt_reset_timeout();
+	}
+
+cleanup:
+	do_cleanup_display(display);
+	igt_remove_fb(display->drm_fd, &fb_info);
+	igt_remove_fb(display->drm_fd, &cursor_fb);
+
+	if (skip_test)
+		igt_skip("Nonblocking modeset is not supported by this kernel\n");
+}
+
 static void two_screens_flip_vs_cursor(igt_display_t *display, int nloops, bool modeset)
 {
 	struct drm_mode_cursor arg[2], arg2[2];
@@ -646,7 +928,7 @@ cleanup:
 		igt_skip("Nonblocking modeset is not supported by this kernel\n");
 }
 
-static void basic_cursor_vs_flip(igt_display_t *display, enum flip_test mode, int nloops)
+static void cursor_vs_flip(igt_display_t *display, enum flip_test mode, int nloops)
 {
 	struct drm_mode_cursor arg[2];
 	struct drm_event_vblank vbl;
@@ -867,6 +1149,155 @@ cleanup:
 		igt_skip("Nonblocking modeset is not supported by this kernel\n");
 }
 
+static void flip_vs_cursor_crc(igt_display_t *display, bool atomic)
+{
+	struct drm_mode_cursor arg[2];
+	struct drm_event_vblank vbl;
+	struct igt_fb fb_info, cursor_fb;
+	unsigned vblank_start;
+	enum pipe pipe = find_connected_pipe(display, false);
+	igt_pipe_crc_t *pipe_crc;
+	igt_crc_t crcs[3];
+
+	if (atomic)
+		igt_require(display->is_atomic);
+
+	igt_require(set_fb_on_crtc(display, pipe, &fb_info));
+
+	igt_create_color_fb(display->drm_fd, 64, 64, DRM_FORMAT_ARGB8888, 0, 1., 1., 1., &cursor_fb);
+	populate_cursor_args(display, pipe, arg, &cursor_fb);
+
+	arg[0].flags = arg[1].flags |= DRM_MODE_CURSOR_BO;
+	arg[1].handle = 0;
+	arg[1].width = arg[1].height = 0;
+
+	igt_display_commit2(display, display->is_atomic ? COMMIT_ATOMIC : COMMIT_LEGACY);
+
+	pipe_crc = igt_pipe_crc_new(pipe, INTEL_PIPE_CRC_SOURCE_AUTO);
+
+	/* Collect reference crc with cursor disabled. */
+	igt_pipe_crc_collect_crc(pipe_crc, &crcs[1]);
+
+	set_cursor_on_pipe(display, pipe, &cursor_fb);
+	igt_display_commit2(display, COMMIT_UNIVERSAL);
+
+	do_ioctl(display->drm_fd, DRM_IOCTL_MODE_CURSOR, &arg[0]);
+
+	/* Collect reference crc with cursor enabled. */
+	igt_pipe_crc_collect_crc(pipe_crc, &crcs[0]);
+
+	/* Disable cursor, and immediately queue a flip. Check if resulting crc is correct. */
+	for (int i = 1; i >= 0; i--) {
+		vblank_start = get_vblank(display->drm_fd, pipe, DRM_VBLANK_NEXTONMISS);
+
+		flip_nonblocking(display, pipe, atomic, &fb_info);
+		do_ioctl(display->drm_fd, DRM_IOCTL_MODE_CURSOR, &arg[i]);
+
+		igt_assert_eq(get_vblank(display->drm_fd, pipe, 0), vblank_start);
+
+		igt_set_timeout(1, "Stuck page flip");
+		igt_ignore_warn(read(display->drm_fd, &vbl, sizeof(vbl)));
+		igt_reset_timeout();
+
+		igt_assert_eq(get_vblank(display->drm_fd, pipe, 0), vblank_start + 1);
+
+		igt_pipe_crc_collect_crc(pipe_crc, &crcs[2]);
+
+		igt_assert_crc_equal(&crcs[i], &crcs[2]);
+	}
+
+	do_cleanup_display(display);
+	igt_remove_fb(display->drm_fd, &fb_info);
+	igt_remove_fb(display->drm_fd, &cursor_fb);
+	igt_pipe_crc_free(pipe_crc);
+}
+
+static void flip_vs_cursor_busy_crc(igt_display_t *display, bool atomic)
+{
+	struct drm_mode_cursor arg[2];
+	struct drm_event_vblank vbl;
+	struct igt_fb fb_info[2], cursor_fb;
+	unsigned vblank_start;
+	enum pipe pipe = find_connected_pipe(display, false);
+	igt_pipe_crc_t *pipe_crc;
+	igt_crc_t crcs[3];
+
+	if (atomic)
+		igt_require(display->is_atomic);
+
+	igt_require(set_fb_on_crtc(display, pipe, &fb_info[0]));
+	igt_create_color_pattern_fb(display->drm_fd, fb_info[0].width, fb_info[0].height,
+				    DRM_FORMAT_XRGB8888, LOCAL_I915_FORMAT_MOD_X_TILED, .1, .1, .1, &fb_info[1]);
+
+	igt_create_color_fb(display->drm_fd, 64, 64, DRM_FORMAT_ARGB8888, 0, 1., 1., 1., &cursor_fb);
+	populate_cursor_args(display, pipe, arg, &cursor_fb);
+
+	arg[0].flags = arg[1].flags |= DRM_MODE_CURSOR_BO;
+	arg[1].handle = 0;
+	arg[1].width = arg[1].height = 0;
+
+	igt_display_commit2(display, display->is_atomic ? COMMIT_ATOMIC : COMMIT_LEGACY);
+
+	pipe_crc = igt_pipe_crc_new(pipe, INTEL_PIPE_CRC_SOURCE_AUTO);
+
+	/* Collect reference crc with cursor disabled. */
+	igt_pipe_crc_collect_crc(pipe_crc, &crcs[1]);
+
+	set_cursor_on_pipe(display, pipe, &cursor_fb);
+	igt_display_commit2(display, COMMIT_UNIVERSAL);
+
+	do_ioctl(display->drm_fd, DRM_IOCTL_MODE_CURSOR, &arg[0]);
+
+	/* Collect reference crc with cursor enabled. */
+	igt_pipe_crc_collect_crc(pipe_crc, &crcs[0]);
+
+	/*
+	  * Set fb 1 on primary at least once before flipping to force
+	  * setting the correct cache level, else we get a stall in the
+	  * page flip handler.
+	  */
+	igt_plane_set_fb(&display->pipes[pipe].planes[IGT_PLANE_PRIMARY], &fb_info[1]);
+	igt_display_commit2(display, COMMIT_UNIVERSAL);
+
+	igt_plane_set_fb(&display->pipes[pipe].planes[IGT_PLANE_PRIMARY], &fb_info[0]);
+	igt_display_commit2(display, COMMIT_UNIVERSAL);
+
+	/* Disable cursor, and immediately queue a flip. Check if resulting crc is correct. */
+	for (int i = 1; i >= 0; i--) {
+		uint32_t *busy;
+
+		busy = make_fb_busy(display->drm_fd, &fb_info[1]);
+
+		vblank_start = get_vblank(display->drm_fd, pipe, DRM_VBLANK_NEXTONMISS);
+
+		flip_nonblocking(display, pipe, atomic, &fb_info[1]);
+		do_ioctl(display->drm_fd, DRM_IOCTL_MODE_CURSOR, &arg[i]);
+
+		igt_assert_eq(get_vblank(display->drm_fd, pipe, 0), vblank_start);
+
+		igt_pipe_crc_collect_crc(pipe_crc, &crcs[2]);
+
+		finish_fb_busy(busy);
+
+		igt_set_timeout(1, "Stuck page flip");
+		igt_ignore_warn(read(display->drm_fd, &vbl, sizeof(vbl)));
+		igt_reset_timeout();
+
+		igt_assert_lte(vblank_start + 1, get_vblank(display->drm_fd, pipe, 0));
+
+		igt_plane_set_fb(&display->pipes[pipe].planes[IGT_PLANE_PRIMARY], &fb_info[0]);
+		igt_display_commit2(display, COMMIT_UNIVERSAL);
+
+		igt_assert_crc_equal(&crcs[i], &crcs[2]);
+	}
+
+	do_cleanup_display(display);
+	igt_remove_fb(display->drm_fd, &fb_info[1]);
+	igt_remove_fb(display->drm_fd, &fb_info[0]);
+	igt_remove_fb(display->drm_fd, &cursor_fb);
+	igt_pipe_crc_free(pipe_crc);
+}
+
 igt_main
 {
 	const int ncpus = sysconf(_SC_NPROCESSORS_ONLN);
@@ -923,6 +1354,12 @@ igt_main
 	igt_subtest("all-pipes-torture-move")
 		stress(&display, -1, -ncpus, DRM_MODE_CURSOR_MOVE, 20);
 
+	igt_subtest("nonblocking-modeset-vs-cursor-atomic")
+		nonblocking_modeset_vs_cursor(&display, 1);
+
+	igt_subtest("long-nonblocking-modeset-vs-cursor-atomic")
+		nonblocking_modeset_vs_cursor(&display, 16);
+
 	igt_subtest("2x-flip-vs-cursor-legacy")
 		two_screens_flip_vs_cursor(&display, 8, false);
 
@@ -947,6 +1384,18 @@ igt_main
 	igt_subtest("2x-long-cursor-vs-nonblocking-modeset-atomic")
 		two_screens_cursor_vs_flip(&display, 150, true);
 
+	igt_subtest("flip-vs-cursor-crc-legacy")
+		flip_vs_cursor_crc(&display, false);
+
+	igt_subtest("flip-vs-cursor-crc-atomic")
+		flip_vs_cursor_crc(&display, true);
+
+	igt_subtest("flip-vs-cursor-busy-crc-legacy")
+		flip_vs_cursor_busy_crc(&display, false);
+
+	igt_subtest("flip-vs-cursor-busy-crc-atomic")
+		flip_vs_cursor_busy_crc(&display, true);
+
 	for (i = 0; i <= flip_test_last; i++) {
 		const char *modes[flip_test_last+1] = {
 			"legacy",
@@ -967,14 +1416,20 @@ igt_main
 		default: break;
 		}
 
-		igt_subtest_f("%sflip-vs-cursor-%s", prefix, modes[i])
-			basic_flip_vs_cursor(&display, i, 0);
-		igt_subtest_f("long-flip-vs-cursor-%s", modes[i])
-			basic_flip_vs_cursor(&display, i, 150);
-		igt_subtest_f("%scursor-vs-flip-%s", prefix, modes[i])
-			basic_cursor_vs_flip(&display, i, 4);
-		igt_subtest_f("long-cursor-vs-flip-%s", modes[i])
-			basic_cursor_vs_flip(&display, i, 150);
+		igt_subtest_f("%sflip-before-cursor-%s", prefix, modes[i])
+			basic_flip_cursor(&display, i, FLIP_BEFORE_CURSOR, 0);
+
+		igt_subtest_f("%sbusy-flip-before-cursor-%s", prefix, modes[i])
+			basic_flip_cursor(&display, i, FLIP_BEFORE_CURSOR,
+					  BASIC_BUSY);
+
+		igt_subtest_f("%sflip-after-cursor-%s", prefix, modes[i])
+			basic_flip_cursor(&display, i, FLIP_AFTER_CURSOR, 0);
+
+		igt_subtest_f("flip-vs-cursor-%s", modes[i])
+			flip_vs_cursor(&display, i, 150);
+		igt_subtest_f("cursor-vs-flip-%s", modes[i])
+			cursor_vs_flip(&display, i, 150);
 
 		igt_subtest_f("cursorA-vs-flipA-%s", modes[i])
 			flip(&display, 0, 0, 10, i);
