@@ -24,6 +24,7 @@
 #include "config.h"
 #endif
 
+#include "igt.h"
 #include <cairo.h>
 #include <errno.h>
 #include <stdint.h>
@@ -31,11 +32,7 @@
 #include <string.h>
 #include <sys/time.h>
 
-#include "drmtest.h"
 #include "intel_bufmgr.h"
-#include "intel_batchbuffer.h"
-#include "intel_io.h"
-#include "igt_kms.h"
 
 #define MAX_CONNECTORS  10
 #define MAX_CRTCS       3
@@ -73,6 +70,8 @@ enum test_flags {
 	TEST_CLONE			= 0x02,
 	TEST_SINGLE_CRTC_CLONE		= 0x04,
 	TEST_EXCLUSIVE_CRTC_CLONE	= 0x08,
+	TEST_STEALING			= 0x10,
+	TEST_TIMINGS			= 0x20,
 };
 
 struct test_config {
@@ -84,7 +83,6 @@ struct test_config {
 struct connector_config {
 	drmModeConnector *connector;
 	int crtc_idx;
-	bool connected;
 	drmModeModeInfo default_mode;
 };
 
@@ -149,8 +147,6 @@ static int paint_fb(struct igt_fb *fb, const char *test_name,
 
 	cr = igt_get_cairo_ctx(drm_fd, fb);
 
-	igt_paint_test_pattern(cr, fb->width, fb->height);
-
 	cairo_move_to(cr, fb->width / 2, fb->height / 2);
 	cairo_set_font_size(cr, 24);
 	igt_cairo_printf_line(cr, align_hcenter, 40, "%s", test_name);
@@ -183,10 +179,11 @@ static void create_fb_for_crtc(struct crtc_config *crtc,
 
 	bpp = 32;
 	depth = 24;
-	fb_id = igt_create_fb(drm_fd, crtc->mode.hdisplay,
-				  crtc->mode.vdisplay,
-				  igt_bpp_depth_to_drm_format(bpp, depth),
-				  LOCAL_DRM_FORMAT_MOD_NONE, fb_info);
+	fb_id = igt_create_pattern_fb(drm_fd, crtc->mode.hdisplay,
+				      crtc->mode.vdisplay,
+				      igt_bpp_depth_to_drm_format(bpp, depth),
+				      LOCAL_DRM_FORMAT_MOD_NONE,
+				      fb_info);
 	igt_assert_lt(0, fb_id);
 }
 
@@ -252,10 +249,9 @@ static void get_crtc_config_str(struct crtc_config *crtc, char *buf,
 		drmModeConnector *connector = crtc->cconfs[i].connector;
 
 		pos += snprintf(&buf[pos], buf_size - pos,
-			"%s%s-%d[%d]%s", i ? ", " : "",
+			"%s%s-%d[%d]", i ? ", " : "",
 			kmstest_connector_type_str(connector->connector_type),
-			connector->connector_type_id, connector->connector_id,
-			crtc->cconfs[i].connected ? "" : " (NC)");
+			connector->connector_type_id, connector->connector_id);
 		if (pos > buf_size)
 			return;
 	}
@@ -271,6 +267,8 @@ static void setup_crtcs(drmModeRes *resources, struct connector_config *cconf,
 	int i;
 	int encoder_usage_count[resources->count_encoders];
 
+	kmstest_unset_all_crtcs(drm_fd, resources);
+
 	i = 0;
 	crtc_count = 0;
 	crtc = crtcs;
@@ -309,9 +307,17 @@ static void setup_crtcs(drmModeRes *resources, struct connector_config *cconf,
 			connector = cconf[i + j].connector;
 
 			/* Intel connectors have only a single encoder */
-			igt_assert(connector->count_encoders == 1);
-			encoder = drmModeGetEncoder(drm_fd,
-						    connector->encoders[0]);
+			if (connector->count_encoders == 1) {
+				encoder = drmModeGetEncoder(drm_fd,
+							    connector->encoders[0]);
+			} else {
+				igt_assert_eq(connector->connector_type,
+					      DRM_MODE_CONNECTOR_DisplayPort);
+
+				igt_assert(connector->count_encoders >= crtc->crtc_idx);
+				encoder = drmModeGetEncoder(drm_fd,
+					connector->encoders[crtc_count]);
+			}
 			igt_assert(encoder);
 
 			config_valid &= !!(encoder->possible_crtcs &
@@ -336,9 +342,13 @@ static void setup_crtcs(drmModeRes *resources, struct connector_config *cconf,
 	for (i = 0; i < connector_count; i++) {
 		drmModeConnector *connector = cconf[i].connector;
 		drmModeEncoder *encoder;
+		int idx = 0;
 
-		igt_assert(connector->count_encoders == 1);
-		encoder = drmModeGetEncoder(drm_fd, connector->encoders[0]);
+		/* DP MST configs are presumed valid */
+		if (connector->count_encoders > 1)
+			idx = cconf[i].crtc_idx;
+
+		encoder = drmModeGetEncoder(drm_fd, connector->encoders[idx]);
 		encoder_usage_count[get_encoder_idx(resources, encoder)]++;
 		drmModeFreeEncoder(encoder);
 	}
@@ -372,6 +382,115 @@ static uint32_t *get_connector_ids(struct crtc_config *crtc)
 	return ids;
 }
 
+static int test_stealing(int fd, struct crtc_config *crtc, uint32_t *ids)
+{
+	int i, ret = 0;
+
+	if (!crtc->connector_count)
+		return drmModeSetCrtc(fd, crtc->crtc_id,
+				     crtc->fb_info.fb_id, 0, 0,
+				     ids, crtc->connector_count, &crtc->mode);
+
+	for (i = 0; i < crtc->connector_count; ++i) {
+		ret = drmModeSetCrtc(fd, crtc->crtc_id,
+				     crtc->fb_info.fb_id, 0, 0,
+				     &ids[i], 1, &crtc->mode);
+
+		igt_assert_eq(ret, 0);
+
+		ret = drmModeSetCrtc(fd, crtc->crtc_id,
+				     crtc->fb_info.fb_id, 0, 0,
+				     ids, crtc->connector_count, &crtc->mode);
+
+		/* This should fail with -EINVAL */
+		if (!ret)
+			return 0;
+	}
+
+	return ret;
+}
+
+static double frame_time(const drmModeModeInfo *kmode)
+{
+	return 1000.0 * kmode->htotal * kmode->vtotal / kmode->clock;
+}
+
+static double line_time(const drmModeModeInfo *kmode)
+{
+	return 1000.0 * kmode->htotal / kmode->clock;
+}
+
+static void check_timings(int crtc_idx, const drmModeModeInfo *kmode)
+{
+#define CALIBRATE_TS_STEPS 120 /* ~2s has to be less than 128! */
+	drmVBlank wait;
+	igt_stats_t stats;
+	uint32_t last_seq;
+	uint64_t last_timestamp;
+	double expected;
+	double accuracy;
+	double mean;
+	double stddev;
+	int n;
+
+	memset(&wait, 0, sizeof(wait));
+	wait.request.type = kmstest_get_vbl_flag(crtc_idx);
+	wait.request.type |= DRM_VBLANK_ABSOLUTE | DRM_VBLANK_NEXTONMISS;
+	do_or_die(drmWaitVBlank(drm_fd, &wait));
+
+	last_seq = wait.reply.sequence;
+	last_timestamp = wait.reply.tval_sec;
+	last_timestamp *= 1000000;
+	last_timestamp += wait.reply.tval_usec;
+
+	memset(&wait, 0, sizeof(wait));
+	wait.request.type = kmstest_get_vbl_flag(crtc_idx);
+	wait.request.type |= DRM_VBLANK_ABSOLUTE | DRM_VBLANK_EVENT;
+	wait.request.sequence = last_seq;
+	for (n = 0; n < CALIBRATE_TS_STEPS; n++) {
+		++wait.request.sequence;
+		do_or_die(drmWaitVBlank(drm_fd, &wait));
+	}
+
+	igt_stats_init_with_size(&stats, CALIBRATE_TS_STEPS);
+	for (n = 0; n < CALIBRATE_TS_STEPS; n++) {
+		struct drm_event_vblank ev;
+		uint64_t now;
+
+		igt_assert(read(drm_fd, &ev, sizeof(ev)) == sizeof(ev));
+		igt_assert_eq(ev.sequence, last_seq + 1);
+
+		now = ev.tv_sec;
+		now *= 1000000;
+		now += ev.tv_usec;
+
+		igt_stats_push(&stats, now - last_timestamp);
+
+		last_timestamp = now;
+		last_seq = ev.sequence;
+	}
+
+	expected = frame_time(kmode);
+
+	mean = igt_stats_get_mean(&stats);
+	stddev = igt_stats_get_std_deviation(&stats);
+	accuracy = 3.090 * stddev; /* 2-tailed 99% confidence */
+
+	igt_info("Expected frametime: %.0fus; measured %.1fus +- %.3fus accuracy %.2f%% [%.2f scanlines]\n",
+		 expected, mean, stddev,
+		 100 * accuracy / mean, accuracy / line_time(kmode));
+
+	/* 99% accuracy within one scanline */
+	igt_assert_f(accuracy < line_time(kmode),
+		     "vblank accuracy (%.3fus, %.1f%%) worse than a scanline (%.3fus)\n",
+		     accuracy, 100 * accuracy / mean, line_time(kmode));
+
+	igt_assert_f(fabs(mean - expected) < 2*stddev,
+		     "vblank interval differs from modeline! expected %.1fus, measured %1.fus +- %.3fus, difference %.1fus (%.1f sigma)\n",
+		     expected, mean, stddev,
+		     fabs(mean - expected), fabs(mean - expected) / stddev);
+}
+
 static void test_crtc_config(const struct test_config *tconf,
 			     struct crtc_config *crtcs, int crtc_count)
 {
@@ -380,7 +499,6 @@ static void test_crtc_config(const struct test_config *tconf,
 	struct crtc_config *crtc;
 	static int test_id;
 	bool config_failed = false;
-	bool connector_connected = false;
 	int ret = 0;
 	int i;
 
@@ -404,7 +522,6 @@ static void test_crtc_config(const struct test_config *tconf,
 
 	for (i = 0; i < crtc_count; i++) {
 		uint32_t *ids;
-		int j;
 
 		crtc = &crtcs[i];
 
@@ -414,24 +531,25 @@ static void test_crtc_config(const struct test_config *tconf,
 		paint_fb(&crtc->fb_info, tconf->name, crtc_strs, crtc_count, i);
 
 		ids = get_connector_ids(crtc);
-		ret = drmModeSetCrtc(drm_fd, crtc->crtc_id,
-				     crtc->fb_info.fb_id, 0, 0, ids,
-				     crtc->connector_count, &crtc->mode);
+		if (tconf->flags & TEST_STEALING)
+			ret = test_stealing(drm_fd, crtc, ids);
+		else
+			ret = drmModeSetCrtc(drm_fd, crtc->crtc_id,
+					     crtc->fb_info.fb_id, 0, 0, ids,
+					     crtc->connector_count, &crtc->mode);
+
 		free(ids);
 
 		if (ret < 0) {
-			igt_assert(errno == EINVAL);
+			igt_assert_eq(errno, EINVAL);
 			config_failed = true;
 		}
-
-		for (j = 0; j < crtc->connector_count; j++)
-			connector_connected |= crtc->cconfs[j].connected;
 	}
 
 	igt_assert(config_failed == !!(tconf->flags & TEST_INVALID));
 
-	if (ret == 0 && connector_connected && !(tconf->flags & TEST_INVALID))
-		sleep(5);
+	if (ret == 0 && tconf->flags & TEST_TIMINGS)
+		check_timings(crtcs[0].crtc_idx, &crtcs[0].mode);
 
 	for (i = 0; i < crtc_count; i++) {
 		if (crtcs[i].fb_info.fb_id) {
@@ -493,37 +611,22 @@ static int get_one_connector(drmModeRes *resources, int connector_id,
 			     struct connector_config *cconf)
 {
 	drmModeConnector *connector;
-	drmModeModeInfo mode;
 
-	connector = drmModeGetConnector(drm_fd, connector_id);
+	connector = drmModeGetConnectorCurrent(drm_fd, connector_id);
 	igt_assert(connector);
 	cconf->connector = connector;
 
-	cconf->connected = connector->connection == DRM_MODE_CONNECTED;
-
-	/*
-	 * For DP/eDP we need a connected sink, since mode setting depends
-	 * on successful link training and retrieved DPCD parameters.
-	 */
-	switch (connector->connector_type) {
-	case DRM_MODE_CONNECTOR_DisplayPort:
-	case DRM_MODE_CONNECTOR_eDP:
-		if (!cconf->connected) {
-			drmModeFreeConnector(connector);
-			return -1;
-		}
+	if (connector->connection != DRM_MODE_CONNECTED) {
+		drmModeFreeConnector(connector);
+		return -1;
 	}
 
-	if (cconf->connected) {
-		if (!kmstest_get_connector_default_mode(drm_fd, connector,
-							&mode))
-			mode = mode_640_480;
-	} else {
-		mode = mode_640_480;
+	if (!kmstest_get_connector_default_mode(drm_fd, connector,
+						&cconf->default_mode)) {
+		drmModeFreeConnector(connector);
+		return -1;
 	}
 
-	cconf->default_mode = mode;
-
 	return 0;
 }
 
@@ -537,7 +640,7 @@ static int get_connectors(drmModeRes *resources, int *connector_idxs,
 		int connector_id;
 
 		connector_idx = connector_idxs[i];
-		igt_assert(connector_idx < resources->count_connectors);
+		igt_assert_lt(connector_idx, resources->count_connectors);
 		connector_id = resources->connectors[connector_idx];
 
 		if (get_one_connector(resources, connector_id, &cconfs[i]) < 0)
@@ -615,6 +718,9 @@ static void test_combinations(const struct test_config *tconf,
 	struct connector_config *cconfs;
 	int i;
 
+	if (connector_count > 2 && (tconf->flags & TEST_STEALING))
+		return;
+
 	get_combinations(tconf->resources->count_connectors, connector_count,
 			 false, &connector_combs);
 	get_combinations(tconf->resources->count_crtcs, connector_count,
@@ -662,7 +768,7 @@ static void run_test(const struct test_config *tconf)
 		test_combinations(tconf, connector_num);
 }
 
-static int opt_handler(int opt, int opt_index)
+static int opt_handler(int opt, int opt_index, void *data)
 {
 	switch (opt) {
 	case 'd':
@@ -684,14 +790,17 @@ int main(int argc, char **argv)
 		enum test_flags flags;
 		const char *name;
 	} tests[] = {
+		{ TEST_TIMINGS, "basic" },
 		{ TEST_CLONE | TEST_SINGLE_CRTC_CLONE,
-					"clone-single-crtc" },
+					"basic-clone-single-crtc" },
 		{ TEST_INVALID | TEST_CLONE | TEST_SINGLE_CRTC_CLONE,
 					"invalid-clone-single-crtc" },
 		{ TEST_INVALID | TEST_CLONE | TEST_EXCLUSIVE_CRTC_CLONE,
 					"invalid-clone-exclusive-crtc" },
 		{ TEST_CLONE | TEST_EXCLUSIVE_CRTC_CLONE,
 					"clone-exclusive-crtc" },
+		{ TEST_INVALID | TEST_CLONE | TEST_SINGLE_CRTC_CLONE | TEST_STEALING,
+					"invalid-clone-single-crtc-stealing" }
 	};
 	const char *help_str =
 	       "  -d\t\tDon't run any test, only print what would be done. (still needs DRM access)\n"
@@ -700,7 +809,7 @@ int main(int argc, char **argv)
 	int ret;
 
 	ret = igt_subtest_init_parse_opts(&argc, argv, "dt:", NULL, help_str,
-					  opt_handler);
+					  opt_handler, NULL);
 	if (ret < 0)
 		return ret == -1 ? 0 : ret;
 
@@ -710,7 +819,7 @@ int main(int argc, char **argv)
 		     "only one of -d and -t is accepted\n");
 
 	igt_fixture {
-		drm_fd = drm_open_any_master();
+		drm_fd = drm_open_driver_master(DRIVER_ANY);
 		if (!dry_run)
 			kmstest_set_vt_graphics_mode();
 
